/*********************************************************************/
/*                                                                   */
/* LiteSTEP Wharf Module SDK 1.3                                     */
/*                                                                   */
/* Copyright (c)1998 Francis Gastellu aka Lone Runner/Aegis          */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* This is a sample Wharf module. It is not supposed to provide any  */
/* functionality but to show how to implement your own Wharf module  */
/*                                                                   */
/* ----------------------------------------------------------------- */


#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stddef.h>
#include <math.h>
#include "ALSCLOCK.H"

#define	PROGRAM_VERSION	"alsClock v1.0"


#ifndef _MSC_VER
#define _MAX_PATH	260
#define _MAX_DIR	3
#define _MAX_DRIVE	265
#define _MAX_FNAME	256
#define _MAX_EXT	256
#else
#define M_PI		3.14159265359
#endif

/* ----------------------------------------------------------------- */
char szAppName[] = "alsClock"; // Name of Application, Window class...
char ini[_MAX_PATH];  // Should be put in LoadSetup(), but for now needed for debugging.

// window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void LoadSetup(void);
void draw_clock(HDC hdc);
void draw_wmclock(HDC hdc);

BOOL init=FALSE;
HWND hMainWnd;                    // main window handle
HWND parent;                      // parent window
wharfDataType wharfData;          // Setup data passed to the DLL
int wndSize;                      // Size of the wharf client


short int no_date, no_second, euro_date, wmclock_mode;
int h_pencolor = 0x42E8FC ,
    m_pencolor = 0x42E8FC,
	s_pencolor = 0x42E8FC,
	d_pencolor = 0x000000,
	date_pencolor = 0xFFFFFF;

int refresh_rate;

HDC		DCBack		= NULL;
HBITMAP	BMBack		= NULL,
		OldBack		= NULL;

HDC		DCDblBuff	= NULL;
HBITMAP	BMDblBuff	= NULL,
		OldDblBuff	= NULL;

VOID FreeDblBuff(VOID)
{
	if(DCDblBuff)
	{
		SelectObject(DCDblBuff, OldDblBuff);
		DeleteDC(DCDblBuff);
	}

	DCDblBuff = NULL;

	if(BMDblBuff)
		DeleteObject(BMDblBuff);

	BMDblBuff = NULL;
}

VOID CreateDblBuff(VOID)
{
	HDC DCBack = GetDC(parent);

	DCDblBuff	= CreateCompatibleDC(DCBack);
	BMDblBuff	= CreateCompatibleBitmap(DCBack, wndSize, wndSize);
	OldDblBuff	= (HBITMAP)SelectObject(DCDblBuff, BMDblBuff);

	ReleaseDC(parent, DCBack);
}

/*********************************************************************/
/* DLL Entry point                                                   */
/*********************************************************************/
__declspec( dllexport ) int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType *wd)
{
    parent = ParentWnd; // Save parent window
    // Duplicate wharfData since the one we get will be destroyed
    memcpy(&wharfData, wd, sizeof(wharfDataType));
    wndSize = 64;
	LoadSetup();

    {    // Register the Window class
        WNDCLASS wc;

        memset(&wc,0,sizeof(wc));
        wc.lpfnWndProc = WndProc;       // our window procedure
        wc.hInstance = dllInst;         // hInstance of DLL
        wc.lpszClassName = szAppName;   // our window class name
		wc.style = CS_DBLCLKS;

        if (!RegisterClass(&wc))
        {
            MessageBox(parent,"Error registering window class",szAppName, MB_OK);
            return 1;
        }
    }


    hMainWnd = CreateWindowEx(
        WS_EX_TRANSPARENT,                          // exstyles
        szAppName,                                  // our window class name
        szAppName,                                  // use description for a window title
        WS_CHILD,                                   // window style
        0, 0,										// position
        wndSize,wndSize,                            // width & height of window
        parent,                                     // parent window (litestep wharf window)
        NULL,                                       // no menu
        dllInst,                                    // hInstance of DLL
        0);                                         // no window creation data

    if (!hMainWnd)
    {
        MessageBox(parent,"Error creating window",szAppName,MB_OK);
        return 1;
    }

		// Create our double buffering DC and Bitmap

	CreateDblBuff();

	SetTimer(hMainWnd, 0, refresh_rate, NULL); // Set a timer for clock

    // Set normal cursor
    SetCursor(LoadCursor(NULL,MAKEINTRESOURCE(IDC_ARROW)));

    // DO NOT REMOVE ! Set magicDWord - required!
    // Used by some modules & litestep internals
    SetWindowLong(hMainWnd,GWL_USERDATA,magicDWord);

    // show the window
    ShowWindow(hMainWnd,SW_SHOWNORMAL);

    return 0;
}

/*********************************************************************/
/* Dll closeup request                                               */
/*********************************************************************/
__declspec( dllexport ) void quitWharfModule(HINSTANCE dllInst)
{
	KillTimer(hMainWnd, 0);

	SelectObject(DCBack, OldBack);

	if(DCBack)
		DeleteDC(DCBack);

	if(BMBack)
		DeleteObject(BMBack);

		// Free our double-buffering handles

	FreeDblBuff();

    DestroyWindow(hMainWnd);                // delete our window
    UnregisterClass(szAppName, dllInst);    // unregister window class
}


/*********************************************************************/
/* Window procedure for our window                                   */
/*********************************************************************/
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_CREATE:		return 0;
        case WM_ERASEBKGND: return 0;
        case WM_PAINT:
			{
				PAINTSTRUCT ps;
				RECT r;

				HDC hdc = DCDblBuff,	// our double-buffering DC ;)
					Orig;				// temporary storage

				if(!init)
				{
					HDC DCParent = GetDC(parent);

					// Save original window contents for transparency

					DCBack	= CreateCompatibleDC(DCParent);
					BMBack	= CreateCompatibleBitmap(DCParent, wndSize, wndSize);

					OldBack = (HBITMAP)SelectObject(DCBack, BMBack);
					BitBlt(DCBack, 0, 0, wndSize, wndSize, DCParent, 0, 0, SRCCOPY);

					ReleaseDC(parent, DCParent);

					init = TRUE;
				}

				// Draw our saved area of the parent window

				BitBlt(hdc, 0, 0, wndSize, wndSize, DCBack, 0, 0, SRCCOPY);

                // Replace this block by your own window drawing

                GetClientRect(hwnd,&r);

				// Draw standard clock
				if (wmclock_mode) draw_wmclock(hdc);
				else draw_clock(hdc);

				// Copy the double-buffered stuff into the original DC

				Orig = BeginPaint(hwnd,&ps);

				BitBlt(Orig, 0, 0, wndSize, wndSize, DCDblBuff, 0, 0, SRCCOPY);

				EndPaint(hwnd, &ps);

				// Et voila
			}
            return 0;
        case WM_KEYDOWN:    // forward keyboard messages to parent window
        case WM_KEYUP:
            PostMessage(parent,message,wParam,lParam);
            return 0;
		// Mouse messages are here to ensure we have a good popup menu behaviour. You may insert
        // your own custom actions
        case WM_RBUTTONUP:
                {
                RECT r;
				char msg[512];
				unsigned int secs, mins, hours, days;

                GetWindowRect(hwnd, &r);
				secs = timeGetTime() / 1000;
				mins = secs / 60;
				hours = mins / 60;
				days = hours / 24;

				secs = secs - (mins * 60);
				mins = mins - (hours * 60);
				hours = hours - (days * 24);		

				sprintf(msg, "Development by Ryan Mullin/Whodini.\r\nAssistance by ArKay.\r\nTiles Contributed by: Cypher Hex, Treman, and Whodini.\r\n%s\r\nUptime: %i Days %2.2d:%2.2d:%2.2d\r\n", ini, days, hours ,mins, secs);
				MessageBox(hwnd, msg, PROGRAM_VERSION, MB_OK);
            	PostMessage(GetParent(GetParent(parent)), 9183, 0, 0);
                }
            return 0;
        case WM_RBUTTONDOWN:
        case WM_LBUTTONDOWN:
        case WM_MBUTTONDOWN:
            PostMessage(GetParent(GetParent(parent)), 9183, 0, 0);
            return 0;
		case WM_LBUTTONDBLCLK:
			{
			char winDir[128];
			GetSystemDirectory(winDir, 128);
			ShellExecute(hwnd, "open", "rundll32.exe", "shell32.dll,Control_RunDLL timedate.cpl", winDir, SW_SHOWNORMAL);
			}
		return 0;
		case WM_TIMER:
			{
			RECT r;
			GetClientRect(hMainWnd, &r);
			InvalidateRect(hMainWnd, &r, TRUE);
		}
		return 0;
    }
    return DefWindowProc(hwnd,message,wParam,lParam);
}


/* Loading the setup from modules.ini */

void LoadSetup(void) {

strcpy(ini, wharfData.alsClock);
strcat(ini, "MODULES.INI");


h_pencolor = GetPrivateProfileInt("alsClock", "Hour_Color", 0x000000, ini);
m_pencolor = GetPrivateProfileInt("alsClock", "Minute_Color", 0x000000, ini);
s_pencolor = GetPrivateProfileInt("alsClock", "Second_Color", 0x0000FF, ini);
d_pencolor = GetPrivateProfileInt("alsClock", "Dot_Color", 0xFFFFFF, ini);
date_pencolor = GetPrivateProfileInt("alsClock","Date_Color",0xFFFFFF, ini);
no_date = GetPrivateProfileInt("alsClock", "No_Date",0,ini);
euro_date = GetPrivateProfileInt("alsClock", "Euro_Date", 0, ini);
refresh_rate = GetPrivateProfileInt("alsClock","Refresh_Rate", 1000, ini);
no_second = GetPrivateProfileInt("alsClock","No_Second",0, ini);
wmclock_mode = GetPrivateProfileInt("alsClock","Wmclock_Mode",0,ini);
}


/* Standard Clock Drawing Mode */

void draw_clock(HDC hdc) {

	SYSTEMTIME tm;

	HPEN hpen;
	HPEN hpenold;
	HFONT hf,oldFont;

	char buf[15];

	double angle;
	int hx, hy, mx, my, sx, sy;


	GetLocalTime(&tm);

	angle = (M_PI / 6.0) * (double) tm.wHour + (M_PI / 360.0) * (double) tm.wMinute;
	hx = (wndSize / 2) +  (int) (17 * sin(angle));
	hy = (wndSize / 2) -  (int) (17 * cos(angle));

	angle = (M_PI / 30.0) * (double) tm.wMinute;
	mx = (wndSize / 2) + (int) (22 * sin(angle));
	my = (wndSize / 2) - (int) (22 * cos(angle));

	angle = (M_PI / 30.0) * (double) tm.wSecond;
	sx = (wndSize / 2) + (int) ((double) 25 * sin(angle));
    sy = (wndSize / 2) - (int) ((double) 25 * cos(angle));


	//seconds

	if (!no_second) {
	hpen=CreatePen(PS_SOLID,1,s_pencolor);
	hpenold = SelectObject(hdc, hpen);

	MoveToEx(hdc, wndSize/2, wndSize/2, NULL);
	LineTo(hdc, sx, sy);
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	}

	// minutes
	
	hpen=CreatePen(PS_SOLID,3,m_pencolor);
	hpenold = SelectObject(hdc, hpen);
	MoveToEx(hdc,wndSize/2, wndSize/2, NULL);
	LineTo(hdc, mx, my);

	//hours
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	hpen=CreatePen(PS_SOLID,3,h_pencolor);
	hpenold = SelectObject(hdc, hpen);

	MoveToEx(hdc, wndSize/2, wndSize/2, NULL);
	LineTo(hdc, hx, hy);


	// DOT
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	hpen=CreatePen(PS_SOLID,3,d_pencolor);
	hpenold = SelectObject(hdc, hpen);

	Ellipse(hdc, (wndSize/2)-1, (wndSize/2)-1, (wndSize/2)+1, (wndSize/2)+1);
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);


	//date
	if (!no_date) {
		int oldBGMode;

		if (euro_date) wsprintf(buf, "%d/%d/%d", tm.wDay, tm.wMonth, tm.wYear-1900);
		else wsprintf(buf, "%d/%d/%d", tm.wMonth, tm.wDay, tm.wYear-1900);
			hf = CreateFont (
             	    8		    // height
 	           	  , 6            // width
 	           	  , 0            // angle escapement
 	           	  , 0		// angle baseline
 	           	  , 0	        // weight
 	           	  , FALSE		 // italic?
 	          	  , FALSE	   // underlined?
 	           	  , FALSE        // strikeout?
             	  , ANSI_CHARSET
 	           	  , OUT_DEFAULT_PRECIS
             	  , CLIP_DEFAULT_PRECIS
 	           	  , DEFAULT_QUALITY
             	  , DEFAULT_PITCH
 	           	  , 0 );
		oldFont = SelectObject(hdc, hf);
		oldBGMode = SetBkMode(hdc, TRANSPARENT);
		SetTextColor(hdc, date_pencolor);
		TextOut( hdc, wndSize/2-25, wndSize/2+10, buf, strlen(buf));
		SelectObject(hdc, oldFont);
		DeleteObject(hf);
		SetBkMode(hdc, oldBGMode);
		}

}

void draw_wmclock(HDC hdc) {

	SYSTEMTIME tm;
	RECT r1 = {5,49,19,59};
	RECT r2 = {23, 49, 59, 59};

	HPEN hpen;
	HPEN hpenold;
	HFONT hf,oldFont;

	char buf[15];

	double angle;
	int hx, hy, mx, my, sx, sy;
	char day [7][3] = {"SU","MO","TU","WE","TH","FR","SA"};
	char month [12][4] = {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};

	GetLocalTime(&tm);

	angle = (M_PI / 6.0) * (double) tm.wHour + (M_PI / 360.0) * (double) tm.wMinute;
	hx = (wndSize/2) +  (int) ((double) 14 * sin(angle));
	hy = 24 -  (int) ((double) 14 * cos(angle));

	angle = (M_PI / 30.0) * (double) tm.wMinute;
	mx = (wndSize/2) + (int) ((double) 18 * sin(angle));
	my = 24 - (int) ((double) 18 * cos(angle));

	angle = (M_PI / 30.0) * (double) tm.wSecond;
	sx = (wndSize/2) + (int) ((double) 20 * sin(angle));
    sy = 24 - (int) ((double) 20 * cos(angle));


	//seconds

	if (!no_second) {
	hpen=CreatePen(PS_SOLID,1,s_pencolor);
	hpenold = SelectObject(hdc, hpen);

	MoveToEx(hdc, wndSize/2, 24, NULL);
	LineTo(hdc, sx, sy);
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	}

	// minutes
	
	hpen=CreatePen(PS_SOLID,3,m_pencolor);
	hpenold = SelectObject(hdc, hpen);
	MoveToEx(hdc,wndSize/2, 24, NULL);
	LineTo(hdc, mx, my);

	//hours
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	hpen=CreatePen(PS_SOLID,3,h_pencolor);
	hpenold = SelectObject(hdc, hpen);

	MoveToEx(hdc, wndSize/2, 24, NULL);
	LineTo(hdc, hx, hy);


	// DOT
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);
	hpen=CreatePen(PS_SOLID,3,d_pencolor);
	hpenold = SelectObject(hdc, hpen);

	Ellipse(hdc, (wndSize/2)-1, 24-1, (wndSize/2)+1, 24+1);
	SelectObject(hdc, hpenold);
	DeleteObject(hpen);


	//date
	if (!no_date) {
		int oldBGMode;

		wsprintf(buf, "%s-%d", month[tm.wMonth-1],tm.wDay);
		hf = CreateFont (
             	    10		    // height
 	           	  , 6            // width
 	           	  , 0            // angle escapement
 	           	  , 0		// angle baseline
 	           	  , 0	        // weight
 	           	  , FALSE		 // italic?
 	          	  , FALSE	   // underlined?
 	           	  , FALSE        // strikeout?
             	  , ANSI_CHARSET
 	           	  , OUT_DEFAULT_PRECIS
             	  , CLIP_DEFAULT_PRECIS
				  , DRAFT_QUALITY
             	  , FIXED_PITCH
 	           	  , 0 );
		oldFont = SelectObject(hdc, hf);
		oldBGMode = SetBkMode(hdc, TRANSPARENT);
		SetTextColor(hdc, date_pencolor);
		DrawText(hdc, day[tm.wDayOfWeek], strlen(day[tm.wDayOfWeek]), &r1, DT_CENTER | DT_END_ELLIPSIS | DT_VCENTER | DT_SINGLELINE);
		SelectObject(hdc, oldFont);
		DeleteObject(hf);

		hf = CreateFont (
             	    10		    // height
 	           	  , 6            // width
 	           	  , 0            // angle escapement
 	           	  , 0		// angle baseline
 	           	  , 900	        // weight
 	           	  , FALSE		 // italic?
 	          	  , FALSE	   // underlined?
 	           	  , FALSE        // strikeout?
             	  , ANSI_CHARSET
 	           	  , OUT_DEVICE_PRECIS
             	  , CLIP_DEFAULT_PRECIS
				  , PROOF_QUALITY
             	  , FIXED_PITCH
 	           	  , "Terminal" );
		oldFont = SelectObject(hdc, hf);
		DrawText(hdc, buf, strlen(buf), &r2, DT_CENTER | DT_END_ELLIPSIS | DT_VCENTER | DT_SINGLELINE);
		SelectObject(hdc, oldFont);
		DeleteObject(hf);
		SetBkMode(hdc, oldBGMode);

		}

}



/* Needed for LCC */
int _stdcall DLLMain(void *hinstDLL, unsigned long dwReason, void *reserved)
{
	return(1);
}

